#!/usr/bin/env python
# coding: utf-8

# # D'une liste de métabolites par pathways à un récupitulatif

# On part donc d'un tableur récapitulatif présentant en premiére ligne le nom des différents pathways retenues aprés filtrage par p-value inférieur à 0.05. Sur chaque colonne le nom des différents métabolites mapper dans cette voie. 
# L'objectifs est de furnir un récapitulatifs des données présentants dans un premier temps dans combien de voies différentes un même métabolites à mapper puis dans un seccond temps un le pourcentage de recouvrement entre 2 voie métaboliques différentes.

### Importing libraries

from random import randint
import numpy as np
import pandas as pd
import openpyxl as xls
import csv


### Variables

Pathways_a_enlever=['Metabolic pathways','Biosynthesis of secondary metabolites','Microbial metabolism in diverse environments','Carbon metabolism','2-Oxocarboxylic acid metabolism','Fatty acid metabolism','Biosynthesis of amino acids','Biosynthesis of nucleotide sugars','Biosynthesis of cofactors','Degradation of aromatic compounds','Carbohydrate metabolism','Energy metabolism','Lipid metabolism','Nucleotide metabolism','Amino acid metabolism','Metabolism of other amino acids','Glycan biosynthesis and metabolism','Metabolism of cofactors and vitamins','Metabolism of terpenoids and polyketides','Biosynthesis of other secondary metabolites','Xenobiotics biodegradation and metabolism','Chemical structure transformation maps','Genetic Information Processing','Transcription','Translation','Folding, sorting and degradation','Replication and repair','Information processing in viruses','Environmental Information Processing','Membrane transport','Signal transduction','Signaling molecules and interaction','Cellular Processes','Transport and catabolism','Cell growth and death','Cellular community - eukaryotes','Cellular community - prokaryotes','Cell motility','Immune system','Organismal Systems','Endocrine system','Circulatory system','Digestive system','Excretory system','Nervous system','Sensory system','Development and regeneration','Aging','Environmental adaptation','Human Diseases','Cancer: overview''Cancer: specific types','Infectious disease: viral','Infectious disease: bacterial','Infectious disease: parasitic','Immune disease','Neurodegenerative disease','Substance dependence','Cardiovascular disease','Endocrine and metabolic disease','Drug resistance: antimicrobial','Drug resistance: antineoplastic','Drug Development','Chronology: Antiinfectives','Chronology: Antineoplastics','Chronology: Nervous system agents','Chronology: Other drugs','Target-based classification: G protein-coupled receptors','Target-based classification: Nuclear receptors','Target-based classification: Ion channels','Target-based classification: Transporters','Target-based classification: Enzymes','Structure-based classification','Skeleton-based classification']

Fichier_a_traiter="C:\\Users\\mumec\\Desktop\\Dossier_gitlab_local\\L100_Ramp_Chebi_IDs_Pathways_mis_en_forme_pour_programme.csv"
Sortie_Resemblance_des_pathways="C:\\Users\\mumec\\Desktop\\Dossier_gitlab_local\\table_de_resemblance_L100_Ramp_Chebi_IDs_Pathways.xlsx"
Sortie_fréquence_des_metabolites="C:\\Users\\mumec\\Desktop\\Dossier_gitlab_local\\frequence_des_metabolites_L100_Ramp_Chebi_IDs_Pathways.xlsx"
Sortie_pathways_de_chaque_metabolites="C:\\Users\\mumec\\Desktop\\Dossier_gitlab_local\\Metabolites_et_leurs_pathways_L100_Ramp_Chebi_IDs_Pathways.xlsx"

### Function initialization

def lecture_colonne(file, n, sep=";"):          # fill opening and recovery
  with open(file,"r") as f:
    r=csv.reader(f, delimiter = sep)
    lignes=list(r)                   
    res=[]
    if (n < len(lignes[0])) and (n >= -len(lignes[0])):
      for l in lignes :
        res.append(l[n].strip())
    return res



def noms_pathways (file, sep=";",p_a_e=Pathways_a_enlever) :
    with open(file,'r') as file_csv:
        csv_reader = csv.reader(file_csv, delimiter = sep)
        liste_nom_pathways=[]
    
        for column in csv_reader:
            liste_nom_pathways.append(column)
        a_filtrer=liste_nom_pathways[0]
    
        i_a_e=[]
        p_retires=[]
        for compteur in range (len(a_filtrer)):
            if a_filtrer[compteur] in p_a_e:
                i_a_e.append(compteur)
                p_retires.append(a_filtrer[compteur])

           
        for retirer in p_retires :
            a_filtrer.remove(retirer)
        return (a_filtrer,i_a_e)
        


def traitement_des_pathways (file,sep=";"):
    Tout_les_pathways,i_e= noms_pathways(file, sep=";") 
    Noms_des_Pathways,i_e=noms_pathways(file, sep=";")

    Nombre_de_pathways_totale=len(Tout_les_pathways) # variable interne

    indice_reel=0
    for p in range (Nombre_de_pathways_totale+len(i_e)):# création d'une variables pour chaque pathways
        
        if (p not in i_e):
            transitoire=lecture_colonne(file, p, sep=";")
            Tout_les_pathways[indice_reel]=transitoire[1:len(transitoire)]
            indice_reel+=1

    for b in range (Nombre_de_pathways_totale):  # on veut garder que les métabolites
        for r in range (len(Tout_les_pathways[b])) :
            if(Tout_les_pathways[b][r]==''):
                indice=b
                Tout_les_pathways[b]=Tout_les_pathways[b][0:r]
                break
                
    for i_path in range (Nombre_de_pathways_totale) :
        Tout_les_pathways[i_path]=[ele for ele in Tout_les_pathways[i_path] if ele!="NA"]

            
    tableau_de_resemblance=np.zeros([Nombre_de_pathways_totale,Nombre_de_pathways_totale+2])

### Nettoyage des données d'entrées 


    P1=[]
    P2=[]
    for ligne in range(Nombre_de_pathways_totale):
        somme_liste=0
        for colonne in range(Nombre_de_pathways_totale) :
            if (ligne==colonne):
                tableau_de_resemblance[ligne,colonne]=len(Tout_les_pathways[colonne])
            else :
                commun=0
                P1=Tout_les_pathways[ligne]
                P2=Tout_les_pathways[colonne]
                for metabolite_recherche in (P1):
                    if (metabolite_recherche in P2):
                            commun+=1
                tableau_de_resemblance[ligne,colonne]=commun
                somme_liste+=commun
        tableau_de_resemblance[ligne,Nombre_de_pathways_totale]=somme_liste
        tableau_de_resemblance[ligne,Nombre_de_pathways_totale+1]=somme_liste/(Nombre_de_pathways_totale-1)
    tableau_approximatif=[]

    for line in tableau_de_resemblance:
        tableau_approximatif.append('  '.join(map(str, line)))
    
    tableau_approximatif=np.array(tableau_approximatif, dtype=object)
    
    ensemble_des_metabolites=[]
    nombre_de_metabolites=[]
    for q in range (Nombre_de_pathways_totale):
        Path=Tout_les_pathways[q]
        for w in range (len(Path)):
            if (Path[w] in ensemble_des_metabolites):
                place=ensemble_des_metabolites.index(Path[w])
                nombre_de_metabolites[place]+=1
            else:
                ensemble_des_metabolites.append(Path[w])
                nombre_de_metabolites.append(1)

    Pathways_de_chaque_metabolites=[]
    for met in range (len(ensemble_des_metabolites)):
        Pathways_de_ce_metabolite=[]
        Le_metabolite=ensemble_des_metabolites[met]
        Pathways_de_ce_metabolite.append(Le_metabolite)
        for avance in range (Nombre_de_pathways_totale):
            if (Le_metabolite in Tout_les_pathways[avance]):
                Pathways_de_ce_metabolite.append(Noms_des_Pathways[avance])
        Pathways_de_chaque_metabolites.append(Pathways_de_ce_metabolite)
                
    
    metabolites_in_multi_Pathways=[] 
    Valeurs_metabolites_in_multi_Pathways=[]
    for g in range (len(nombre_de_metabolites)) :
        if nombre_de_metabolites[g]>1 :
            metabolites_in_multi_Pathways.append(ensemble_des_metabolites[g])
            Valeurs_metabolites_in_multi_Pathways.append(nombre_de_metabolites[g])
    Frequences_des_metabolites=np.array([ensemble_des_metabolites, nombre_de_metabolites,metabolites_in_multi_Pathways, Valeurs_metabolites_in_multi_Pathways],dtype=object)    
    return(Noms_des_Pathways,tableau_approximatif,Frequences_des_metabolites , Pathways_de_chaque_metabolites)


def traitement_totale_couverture_pathways_et_métabolites(file,Fichier1,Fichier2,Fichier3):
    Noms_des_Pathways,tableau_de_resemblance,Frequences_des_metabolites , Les_metabolites_et_leurs_pathways = traitement_des_pathways (file,sep=";")    
    
    Les_metabolites_et_leurs_pathways = pd.DataFrame(data=Les_metabolites_et_leurs_pathways)
    
    Noms_des_Pathways.append("recouvrement_totale")
    Noms_des_Pathways.append("recouvrement_moyen")
    resemblance_des_pathways=[Noms_des_Pathways]
    for indice in range(len(tableau_de_resemblance)):
        resemblance_des_pathways.append(tableau_de_resemblance[indice])
        
    resemblance_des_pathways = np.array(resemblance_des_pathways,dtype=object)
    resemblance_des_pathways = pd.DataFrame(data=resemblance_des_pathways)
    
    Frequences_des_metabolites = pd.DataFrame(data=Frequences_des_metabolites)
 
    excel_file1 = pd.ExcelWriter(Fichier1)
    resemblance_des_pathways.to_excel(excel_file1)
    excel_file1.close()

    excel_file2 = pd.ExcelWriter(Fichier2)
    Frequences_des_metabolites.to_excel(excel_file2)
    excel_file2.close()

    excel_file3 = pd.ExcelWriter(Fichier3)
    Les_metabolites_et_leurs_pathways.to_excel(excel_file3)
    excel_file3.close()

###Ligne de commande pour obtenir les résultats : nécessite 1 fichier de résultat et 3 noms des fichiers de sorties

traitement_totale_couverture_pathways_et_métabolites(Fichier_a_traiter,Sortie_Resemblance_des_pathways,Sortie_fréquence_des_metabolites,Sortie_pathways_de_chaque_metabolites)

